@@ -27,6 +27,16 @@ namespace Clasp {
 struct SharedMinimizeData;
 struct ProblemStats;
 
+enum ExitCode {
+	E_UNKNOWN       = 0,       /*!< satisfiablity of problem not knwon; search not started     */
+	E_INTERRUPT     = 1,       /*!< run was interrupted                                        */
+	E_SAT           = 10,      /*!< at least one model was found                               */
+	E_EXHAUST       = 20,      /*!< search-space was completely examined                       */
+	E_MEMORY        = 33,      /*!< run was interrupted by out of memory exception             */
+	E_ERROR         = 65,      /*!< run was interrupted by internal error                      */
+	E_NO_RUN        = 128      /*!< search not started because of syntax or command line error */
+};
+
 /*!
  * Base class for printing status and input format dependent information,
  * like models, optimization values, summaries, and program statistics.
@@ -54,7 +64,7 @@ public:
 			result_sat     = 2, /**< The problem is satisfiable (has at least one model) */
 			result_optimum = 3, /**< The optimization problem is satisfiable and the optimal model was found */
 			result_sat_opt = 4, /**< The optimization problem is satisfiable but the optimal model was not found */
-			not_a_result=5 
+			not_a_result   = 5 
 		};
 		RunSummary(const SharedContext& a_ctx) : ctx(a_ctx) {}
 		const SharedContext& ctx;       /**< context object of the run */ 
@@ -70,6 +80,7 @@ public:
 		int    termSig()const { return sig != 0 ? (int)termId : 0; }
 		int    winner() const { return sig == 0 ? (int)termId : -1;}
 		bool   complete()const{ return sig == 0 && comp != 0; }
+		int    exitCode()const;
 		Result result() const;
 		uint64 models() const;
 	private: RunSummary& operator=(const RunSummary&);
