@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2011, Benjamin Kaufmann
+// Copyright (c) 2006-2012, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -26,7 +26,7 @@
 #endif
 
 #include <clasp/solver_types.h>
-
+#include <clasp/util/atomic.h>
 namespace Clasp { 
 
 //! An array of literals that can be shared between threads.
@@ -93,7 +93,7 @@ public:
 	//! Sets the initial activity for the to be created clause.
 	void setActivity(uint32 a)        { extra_.setActivity(a);  }
 	//! Sets the initial literal block distance for the to be created clause.
-	void setLbd(uint32 l1, uint32 l2) { extra_.setLbd(l1, l2); }
+	void setLbd(uint32 lbd)           { extra_.setLbd(lbd); }
 	//! Discards the current clause.
 	void clear() { literals_.clear(); }
 	
@@ -179,19 +179,16 @@ public:
 	void simplify();
 	
 	//! A type for storing the result of a clause insertion operation.
-	/*!
-	 * The insertion operations return a tuple r s.th.
-	 *   - r.ok is true if the clause was successfully created/integrated.
-	 *   - r.ok is false if the clause is conflicting w.r.t the current assignment.
-	 *   - r.local is the newly created local representation of the clause
-	 */
 	struct Result {
-		explicit Result(ClauseHead* loc = 0, bool o = true)
+		explicit Result(ClauseHead* loc = 0, Status st = status_open)
 			: local(loc)
-			, ok(o) {}
+			, status(st) {}
 		ClauseHead*     local;
-		bool            ok;
-		operator bool() const { return ok; }
+		Status          status;
+		//! Returns false is clause is conflicting w.r.t current assignment.
+		bool     ok()   const { return status != status_conflicting; }
+		bool     unit() const { return (status & status_unit) != 0; }
+		operator bool() const { return ok(); }
 	};
 
 	//! Adds the clause to the solver if possible.
@@ -220,13 +217,14 @@ public:
 		
 	//! Flags controlling clause creation and integration.
 	enum CreateFlag {
-		clause_no_add        = 1, /**< do not add clause to solver db */
-		clause_explicit      = 2, /**< force creation of explicit clause even if size <= 3 */
-		clause_not_sat       = 4, /**< integrate only if clause is not satisfied w.r.t current assignment */
-		clause_not_root_sat  = 8, /**< integrate only if clause is not satisfied w.r.t root level */
-		clause_not_conflict  = 16,/**< integrate only if clause is not conflicting */
-		clause_no_release    = 32,/**< do not call release on shared literals */
-		clause_known_order   = 64,/**< assume clause is already ordered w.r.t watches */
+		clause_no_add        = 1,  /**< do not add clause to solver db */
+		clause_explicit      = 2,  /**< force creation of explicit clause even if size <= 3 */
+		clause_not_sat       = 4,  /**< integrate only if clause is not satisfied w.r.t current assignment */
+		clause_not_root_sat  = 8,  /**< integrate only if clause is not satisfied w.r.t root level */
+		clause_not_conflict  = 16, /**< integrate only if clause is not conflicting */
+		clause_no_release    = 32, /**< do not call release on shared literals */
+		clause_int_lbd       = 64, /**< compute lbd when integrating asserting clauses */
+		clause_known_order   = 128,/**< assume clause is already ordered w.r.t watches */
 	};
 
 	//! Creates a clause from the literals given in lits.
@@ -236,10 +234,8 @@ public:
 	 * \param flags Flag set controlling creation (see CreateFlag).
 	 * \param info  Initial information (e.g. type) for the new clause.
 	 *
-	 * \pre !s.hasConflict() and s.decisionLevel() == 0 or extra.type() != Constraint_t::static_constraint
+	 * \pre !s.hasConflict() and s.decisionLevel() == 0 or extra.learnt()
 	 * \pre lits does not contain duplicate/complementary literals
-	 * \pre lits is empty or lits[0] is either free or true
-	 * \pre if lits[1] is false all lits except for lits[0] are false and no literal was assigned on a higher dl
 	 *
 	 * \note 
 	 *   If the given clause is unit (or asserting), the unit-resulting literal is
@@ -259,6 +255,8 @@ public:
 	static Result create(Solver& s, LitVec& lits, uint32 flags = 0, const ClauseInfo& info = ClauseInfo());
 	/*!
 	 * \overload Result ClauseCreator::create(Solver& s, LitVec& lits, uint32 flags, const ClauseInfo& info);
+	 * \pre lits is empty or lits[0] is either free or true
+	 * \pre if lits[1] is false all lits[i], i > 1, are false and no literal was assigned on a higher dl than lits[1]
 	 * \note 
 	 *   The function always assumes clause_known_order, i.e. it is the caller's job
 	 *   to guarantee the first two literals in lits are valid watches.
@@ -273,7 +271,7 @@ public:
 	 * \param s           The solver in which the clause should be integrated.
 	 * \param clause      The clause to be integrated.
 	 * \param modeFlags   A set of flags controlling integration (see CreateFlag).
-	 * \param e           Initial data for the local representation.
+	 * \param t           Constraint type to use for the local representation.
 	 * 
 	 * \note 
 	 *   The function behaves similar to ClauseCreator::create() with the exception that
@@ -298,7 +296,7 @@ public:
 	 *     return r;
 	 *   \endcode
 	 */
-	static Result integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags, const ClauseInfo& e);
+	static Result integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags, ConstraintType t);
 	/*!
 	 * \overload Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags)
 	 */
@@ -311,11 +309,12 @@ private:
 	static inline Status status(const Solver& s, const Watches& w, uint32 modeFlags);
 	static ClauseHead* newProblemClause(Solver& s, LitVec& lits, const ClauseInfo& e, uint32 flags);
 	static ClauseHead* newLearntClause(Solver& s,  LitVec& lits, const ClauseInfo& e, uint32 flags);
-	Solver*         solver_;    // solver in which new clauses are stored
-	LitVec          literals_;  // literals of the new clause
-	ClauseInfo      extra_;     // extra info 
-	uint32          fwLev_;     // decision level of the first watched literal
-	uint32          swLev_;     // decision level of the second watched literal
+	static ClauseHead* newUnshared(Solver& s, SharedLiterals* clause, const Watches& w, const ClauseInfo& e);
+	Solver*    solver_;    // solver in which new clauses are stored
+	LitVec     literals_;  // literals of the new clause
+	ClauseInfo extra_;     // extra info 
+	uint32     fwLev_;     // decision level of the first watched literal
+	uint32     swLev_;     // decision level of the second watched literal
 };
 
 //! Class for representing a clause in a solver.
@@ -323,7 +322,18 @@ class Clause : public ClauseHead {
 public:
 	typedef Constraint::PropResult PropResult;
 	enum { MAX_SHORT_LEN = 5 };
+	static void* alloc(Solver& s, uint32 lits, bool learnt);
 	
+	//! Creates a new clause object in mem.
+	/*!
+	 * \pre mem points to a memory block that was allocated via Clause::alloc(s, size)
+	 */
+	static ClauseHead*  newClause(void* mem, Solver& s, const Literal* lits, uint32 size, const ClauseInfo& extra);
+
+	static ClauseHead*  newClause(Solver& s, const Literal* lits, uint32 size, const ClauseInfo& extra) {
+		return newClause(alloc(s, size, extra.learnt()), s, lits, size, extra);
+	}
+
 	/*!
 	 * Creates a new problem clause from the literals in lits.
 	 * 
@@ -340,8 +350,7 @@ public:
 	static ClauseHead*  newClause(Solver& s, const LitVec& lits) {
 		return newClause(s, &lits[0], static_cast<uint32>(lits.size()), ClauseInfo());
 	}
-	static ClauseHead*  newClause(Solver& s, const Literal* lits, uint32 size, const ClauseInfo& extra);
-
+	
 	/*!
 	 * Creates a new learnt clause from the literals in lits.
 	 * 
@@ -376,13 +385,6 @@ public:
 	 */ 
 	static ClauseHead*  newContractedClause(Solver& s, const LitVec& lits, const ClauseInfo& extra, LitVec::size_type tail, bool extend);
 
-	//! Creates a new specialized short clause object.
-	/*!
-	 * \pre size <= MAX_SHORT_LEN
-	 * \pre mem is either 0 or points to a memory block that was allocated via s.allocSmall();
-	 */
-	static ClauseHead*  newShortClause(Solver& s, void* mem, const Literal* lits, uint32 size, const ClauseInfo& extra);
-
 	//! Creates a new local surrogate for shared_lits to be used in the given solver.
 	/*!
 	 * \param s The solver in which this clause will be used.
@@ -420,12 +422,8 @@ public:
 	
 	// LearntConstraint interface
 
-	//! Returns true if the clause is currently satisfied.
-	/*!
-	 * If the clause is currently not satisfied all free literals of
-	 * the clause are copied to freeLits.
-	 */
-	bool isSatisfied(const Solver& s, LitVec& freeLits);
+	//! Returns type() if the clause is currently not satisfied and t.inSet(type()).
+	uint32 isOpen(const Solver& s, const TypeSet& t, LitVec& freeLits);
 	
 	// clause interface
 	BoolPair strengthen(Solver& s, Literal p, bool allowToShort);
@@ -434,11 +432,12 @@ public:
 	void     toLits(LitVec& out)    const;
 	bool     contracted()           const { return data_.local.contracted(); }
 	bool     isSmall()              const { return data_.local.isSmall(); }
+	bool     strengthened()         const { return data_.local.strengthened(); }
+	uint32   computeAllocSize()     const;
 private:
 	Clause(Solver& s, const ClauseInfo& extra, const Literal* lits, uint32 size, uint32 tail, bool extend);
 	Clause(Solver& s, const Clause& other);
 	typedef std::pair<Literal*, Literal*> LitRange;
-	static void* alloc(Solver& s, uint32 lits);
 	void         undoLevel(Solver& s);
 	bool         updateWatch(Solver& s, uint32 pos);
 	Literal*     removeFromTail(Solver& s, Literal* it, Literal* end);
@@ -491,10 +490,7 @@ public:
 	 *
 	 * \pre all body-literals are currently false.
 	 */ 
-	static LoopFormula* newLoopFormula(Solver& s, Literal* bodyLits, uint32 numBodies, uint32 bodyToWatch, uint32 numAtoms, const ClauseInfo& extra = ClauseInfo()) {
-		void* mem = ::operator new( sizeof(LoopFormula) + (numBodies+numAtoms+3) * sizeof(Literal) );
-		return new (mem) LoopFormula(s, numBodies+numAtoms, bodyLits, numBodies, bodyToWatch, extra);
-	}
+	static LoopFormula* newLoopFormula(Solver& s, Literal* bodyLits, uint32 numBodies, uint32 bodyToWatch, uint32 numAtoms, const Activity& act);
 
 	//! Adds an atom to the loop-formula.
 	/*!
@@ -519,35 +515,28 @@ public:
 	// LearntConstraint interface
 	bool locked(const Solver& s) const;
 	
-	//! Returns true if the loop-formula is satisfied.
-	/*!
-	 * A loop-formula is satisfied if one body is true or all contained atoms are false.
-	 * If the loop-formula is currently not satisfied isSatisfied copies all free literals
-	 * to freeLits.
-	 */
-	bool isSatisfied(const Solver& s, LitVec& freeLits);
+	uint32 isOpen(const Solver& s, const TypeSet& t, LitVec& freeLits);
 	
 	//! Returns the loop-formula's activity.
 	/*!
 	 * The activity of a loop-formula is increased, whenever reason() is called.
 	 */
-	Activity activity() const { return makeActivity(activity_, lbd_); }
+	Activity activity() const { return act_; }
 	
 	//! Halves the loop-formula's activity.
-	void decreaseActivity() { activity_ >>= 1; }
+	void decreaseActivity() { act_ = Activity(act_.activity()>>1, act_.lbd()); }
 	
 	//! Returns Constraint_t::learnt_loop.
 	ConstraintType type() const { return Constraint_t::learnt_loop; }
 private:
-	LoopFormula(Solver& s, uint32 size, Literal* bodyLits, uint32 numBodies, uint32 bodyToWatch, const ClauseInfo& info);
+	LoopFormula(Solver& s, uint32 size, Literal* bodyLits, uint32 numBodies, uint32 bodyToWatch, const Activity& a);
 	bool watchable(const Solver& s, uint32 idx);
 	bool isTrue(const Solver& s, uint32 idx);
-	uint32  activity_:27; // activity of this loop formula
-	uint32  lbd_     : 5; // lbd of active clause
-	uint32  end_;         // position of second sentinel
-	uint32  size_;        // size of lits_
-	uint32  other_;       // stores the position of a literal that was recently true
-	Literal lits_[0];     // S B1...Bm ai S a1...an
+	Activity act_;     // Activity of constraint 
+	uint32   end_;     // position of second sentinel
+	uint32   size_;    // size of lits_
+	uint32   other_;   // stores the position of a literal that was recently true
+	Literal  lits_[0]; // S B1...Bm ai S a1...an
 };
 
 namespace mt {
@@ -579,7 +568,7 @@ public:
 	bool           isReverseReason(const Solver& s, Literal p, uint32 maxL, uint32 maxN);
 	bool           simplify(Solver& s, bool);
 	void           destroy(Solver* s, bool detach);
-	bool           isSatisfied(const Solver& s, LitVec& freeLits);
+	uint32         isOpen(const Solver& s, const TypeSet& t, LitVec& freeLits);
 	uint32         size() const;
 	void           toLits(LitVec& out) const;
 private:
