@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2010, Benjamin Kaufmann
+// Copyright (c) 2006-2012, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -25,6 +25,8 @@
 #endif
 #include <clasp/literal.h>
 #include <clasp/constraint.h>
+#include <clasp/util/misc_types.h>
+#include <clasp/util/atomic.h>
 
 namespace Clasp { 
 class  Solver;
@@ -38,7 +40,7 @@ class  SatPreprocessor;
  */
 //@{
 
-//! Interface for enumerating models
+//! Interface for enumerating models.
 /*!
  * Enumerators are global w.r.t to one search operation, that is
  * even if the search operation itself is a parallel search, there
@@ -46,50 +48,37 @@ class  SatPreprocessor;
  * to protect the enumerator with appropriate locking.
  *
  * Concrete enumerators may create and install an enumeration constraint 
- * in each solver to support (parallel) enumeration
+ * in each solver to support (parallel) enumeration.
  */
 class Enumerator {
 public:
-	enum ProgressType    { progress_restart, progress_reduce, progress_model, num_progress_types };
-	//! Interface used by the enumerator to report restart events
-	class ProgressReport {
-	public:
-		ProgressReport();
-		virtual ~ProgressReport();
-		//! preprocessing progress
-		virtual void reportPreProgress(char /* type */, const SatPreprocessor& /* p */, uint32 /* min */, uint32 /* max */) {}
-		//! solving progress
-		virtual void reportProgress(ProgressType /* t */, const Solver& /* s */, uint64 /* maxCfl */, uint32 /* maxL */) {}
-	private:
-		ProgressReport(const ProgressReport&);
-		ProgressReport& operator=(const ProgressReport&);
-	};
-	//! Interface used by the enumerator to report model events
+	//! Interface used by the Enumerator to report model events.
 	class Report : public ProgressReport {
 	public:
 		Report();
-		//! the solver has found a new model
+		//! The solver has found a new model.
 		virtual void reportModel(const Solver& /* s */, const Enumerator& /* self */) {}
-		//! enumeration has terminated
+		//! Enumeration has terminated.
 		virtual void reportSolution(const Enumerator& /* self */, bool /* complete */) {}
 	};
-	//! A solver-local (i.e. thread-local) constraint to support enumeration
+	//! A solver-local (i.e. thread-local) constraint to support enumeration.
 	class EnumeratorConstraint : public Constraint {
 	public:
+		using Constraint::minimize;
 		EnumeratorConstraint();
-		//! attaches to minimize constraint to use during enumeration
+		//! Attaches to minimize constraint to use during enumeration.
 		void   attach(Solver& s);
-		//! detaches and destroys minimize constraint if any
+		//! Detaches and destroys minimize constraint if any.
 		void   destroy(Solver* s, bool detach);
 		MinimizeConstraint* minimize()  const { return mini_; }
 		uint32 updates() const      { return updates_; }
 		void   setUpdates(uint32 u) { updates_ = u; }
 	protected:
-		//! activates minimize constraint on assignment of tag literal
+		//! Activates minimize constraint on assignment of tag literal.
 		PropResult propagate(Solver&, Literal, uint32&);
-		//! restores minimize constraint on undo of tag literal
+		//! Restores minimize constraint on undo of tag literal.
 		void       undoLevel(Solver&);
-		//! noop
+		//! Noop.
 		void       reason(Solver&, Literal, LitVec&);
 	private:
 		MinimizeConstraint* mini_;
@@ -99,63 +88,61 @@ public:
 	explicit Enumerator(Report* r = 0);
 	virtual ~Enumerator();
 	
-	//! How to continue after a model was found
+	//! How to continue after a model was found.
 	enum Result {
-		enumerate_continue,      /**< continue enumeration */
-		enumerate_stop_complete, /**< stop because search space is exceeded  */
-		enumerate_stop_enough    /**< stop because enough models have been enumerated */
+		enumerate_continue,      /**< Continue enumeration. */
+		enumerate_stop_complete, /**< Stop because search space is exceeded.  */
+		enumerate_stop_enough    /**< Stop because enough models have been enumerated. */
 	};
 
-	//! sets the report callback to be used during enumeration
+	//! Sets the report callback to be used during enumeration.
 	/*!
 	 * \note Ownership is *not* transferred and r must be valid
-	 * during complete search
+	 * during complete search.
 	 */
 	void setReport(Report* r);
-	//! enables progress reporting via the given report callback
-	void enableProgressReport(ProgressReport* r);
 
-	//! if true, does a search-restart every time a model is found
+	//! If true, does a search-restart every time a model is found.
 	void setRestartOnModel(bool r);
 	
-	//! starts initialization of this enumerator
+	//! Starts initialization of this enumerator.
 	/*!
 	 * Must be called once before search is started and before solvers
 	 * can be attached. Shall freeze relevant variables.
-	 * \note In the incremental setting, startInit() must be called once for each incremental step
+	 * \note In the incremental setting, startInit() must be called once for each incremental step.
 	 */
 	void startInit(SharedContext& ctx);
 
-	//! sets the (shared) minimize object to be used during enumeration
+	//! Sets the (shared) minimize object to be used during enumeration.
 	/*!
 	 * \note Ownership is transferred.
 	 */
 	void setMinimize(SharedMinimizeData* min);
 	const SharedMinimizeData* minimize() const { return mini_; }
 
-	//! sets the maximum number of models to enumerate
+	//! Sets the maximum number of models to enumerate.
 	/*!
-	 * \param numModels number of models to enumerate (0 means all models)
+	 * \param numModels Number of models to enumerate (0 means all models).
 	 */
 	void enumerate(uint64 numModels);
 
 	bool enumerate() const { return numModels_ != 1; }
 
-	//! completes the initialization of this enumerator
+	//! Completes the initialization of this enumerator.
 	/*!
 	 * Sets the number of solvers sharing this enumerator.
 	 */
 	EnumeratorConstraint* endInit(SharedContext& ctx, uint32 shareCount);
-	//! force termination of model enumeration in backtrackFromModel()
+	//! Force termination of model enumeration in backtrackFromModel().
 	void   terminate()        { fetch_and_or(updates_, uint32(1)); }
 	bool   terminated() const { return (updates_ & 1u) != 0; }
 
-	uint64 enumerated; /**< Number of models enumerated so far */
+	uint64 enumerated; /**< Number of models enumerated so far. */
 
-	//! returns the enumeration constraint attached to s
+	//! Returns the enumeration constraint attached to s.
 	EnumeratorConstraint* constraint(const Solver& s) const;
 
-	//! processes and returns from a model stored in the given solver
+	//! Processes and returns from a model stored in the given solver.
 	/*!
 	 * The return value determines how search should proceed.
 	 * If enumerate_continue is returned, the enumerator has
@@ -173,7 +160,7 @@ public:
 	 *       at most one solver shall call the function at any one time.
 	 */
 	Result backtrackFromModel(Solver& s, bool callContinue = true);
-	//! continues enumeration after a model was found
+	//! Continues enumeration after a model was found.
 	/*!
 	 * Adds new information from the recent model and strengthens
 	 * any conditional knowledge in s.
@@ -186,7 +173,7 @@ public:
 	 */
 	bool continueFromModel(Solver& s, bool allowModelHeuristic = true);
 
-	//! updates solver s with new model-related information
+	//! Updates solver s with new model-related information.
 	/*!
 	 * The function is used to transfer information in
 	 * parallel search. Whenever a solver s1 found a new model
@@ -197,29 +184,22 @@ public:
 	 */
 	bool update(Solver& s, bool disjointPath);
 
-	//! called once after search has stopped
+	//! Called once after search has stopped.
 	/*!
-	 * The function notifies the installed Report object
+	 * The function notifies the installed Report object.
 	 */
 	void reportResult(bool complete) {
 		if (report_) report_->reportSolution(*this, complete);
 	}
-
-	void reportPreProgress(char type, const SatPreprocessor& p, uint32 min, uint32 max) {
-		if (progress_) progress_->reportPreProgress(type, p, min, max); 
-	}
-
-	void reportProgress(ProgressType t, const Solver& s, uint64 maxCfl, uint32 maxL) {
-		if (progress_) progress_->reportProgress(t, s, maxCfl, maxL); 
-	}
-	
-	//! returns true if optimization is active
+	//! Returns true if optimization is active.
 	bool   optimize() const;
 	bool   optimizeHierarchical() const;
-	//! returns true if there is a next level to optimize
+	//! Returns true if there is a next level to optimize.
 	bool   optimizeNext() const;
-	//! returns whether or not this enumerator supports full restarts once a model was found
+	//! Returns whether or not this enumerator supports full restarts once a model was found.
 	virtual bool supportsRestarts() const{ return true; }
+	//! Returns whether or not this enumerator supports parallel search.
+	virtual bool supportsParallel() const{ return true; }
 protected:
 	uint32 getHighestActiveLevel() const { return activeLevel_; }
 	uint32 nextUpdate()                  { return (updates_ += 2); }
@@ -233,7 +213,6 @@ private:
 	Enumerator& operator=(const Enumerator&);
 	uint64              numModels_;
 	Report*             report_;
-	ProgressReport*     progress_;
 	SharedMinimizeData* mini_;
 	std::atomic<uint32> updates_;
 	uint32              activeLevel_;
