@@ -141,8 +141,9 @@ private:
 		uint32 ufs    :  1; // in ufs-queue?
 		uint32 validS :  1; // is source valid?
 	};
-	// Watch-structure used to update extended bodies affected by literal assignments
+	// Watch-structure used to update extended rules affected by literal assignments
 	struct ExtWatch {
+		enum Type { watch_choice_false = 0, watch_body_goal_false = 1 };
 		NodeId bodyId;
 		uint32 data;
 	};
@@ -259,7 +260,7 @@ private:
 	typedef PodVector<BodyData>::type BodyVec;
 	typedef PodVector<ExtData*>::type ExtVec;
 	typedef PodVector<ExtWatch>::type WatchVec;
-// -------------------------------------------------------------------------------------------  
+	// -------------------------------------------------------------------------------------------  
 	IdQueue          todo_;        // ids of atoms that recently lost their source
 	IdQueue          unfounded_;   // ids of atoms that are unfounded wrt the current assignment (limited to one scc)
 	AtomVec          atoms_;       // data for each atom       
@@ -268,7 +269,8 @@ private:
 	WatchVec         watches_;     // watches for handling choice-, cardinality- and weight rules
 	VarVec           sourceQ_;     // source-pointer propagation queue
 	VarVec           invalid_;     // ids of bodies that became invalid during unit propagation - they can no longer be sources
-	VarVec           pickedExt_;   // extended bodies visited during reason computation
+	VarVec           invalidExt_;  // ids of extended watches that fired during unit propagation
+	VarVec           reasonExt_;   // temporary vector for handling extended rules in computeReason()
 	LitVec           loopAtoms_;   // only used if strategy_ == shared_reason
 	Solver*          solver_;      // my solver
 	DependencyGraph* graph_;       // PBADG
