@@ -580,7 +580,7 @@ uint32 Clause::computeAllocSize() const {
 	if (nw != 0u) {
 		const Literal* eoc = head_ + sz;
 		do { nw -= eoc++->watched(); } while (nw); 
-		sz = eoc - head_;
+		sz = static_cast<uint32>(eoc - head_);
 	}
 	return rt + (sz*sizeof(Literal));
 }
