@@ -335,7 +335,6 @@ void ClaspVmtf::startInit(const Solver& s) {
 
 void ClaspVmtf::endInit(Solver& s) {
 	if (s.strategies().heuReinit)     { vars_.clear(); }
-	if (s.strategies().heuOther == 3) { s.strategies().heuOther = 0; }
 	bool moms = s.strategies().heuMoms != 0;
 	for (Var v = 1; v <= s.numVars(); ++v) {
 		if (s.value(v) == value_free && score_[v].pos_ == vars_.end()) {
@@ -369,7 +368,8 @@ void ClaspVmtf::newConstraint(const Solver& s, const Literal* first, LitVec::siz
 	if (t != Constraint_t::static_constraint) {
 		LessLevel comp(s, score_);
 		VarVec::size_type maxMove = t == Constraint_t::learnt_conflict ? MOVE_TO_FRONT : MOVE_TO_FRONT/2;
-		const bool upAct = t == Constraint_t::learnt_conflict || s.strategies().heuOther >= uint32(t-1);
+		const bool upAct = (t == Constraint_t::learnt_conflict)
+		                || (((s.strategies().heuOther + 1) & 3u) > uint32(t-1));
 		for (LitVec::size_type i = 0; i < size; ++i, ++first) {
 			Var v = first->var(); 
 			score_[v].occ_ += 1 - (((int)first->sign())<<1);
@@ -459,7 +459,6 @@ void ClaspVsids::startInit(const Solver& s) {
 
 void ClaspVsids::endInit(Solver& s) {
 	if (s.strategies().heuReinit)     { vars_.clear(); }
-	if (s.strategies().heuOther == 3) { s.strategies().heuOther = 0; }
 	bool moms = s.strategies().heuMoms != 0;
 	inc_ = 1.0;
 	double maxS = 0.0;
@@ -506,8 +505,8 @@ void ClaspVsids::normalize() {
 
 void ClaspVsids::newConstraint(const Solver& s, const Literal* first, LitVec::size_type size, ConstraintType t) {
 	if (t != Constraint_t::static_constraint) {
-		const uint32 other = s.strategies().heuOther != 3 ? s.strategies().heuOther : 0;
-		const bool upAct   = t == Constraint_t::learnt_conflict || (other >= uint32(t-1));
+		const bool upAct = (t == Constraint_t::learnt_conflict)
+		                || (((s.strategies().heuOther + 1) & 3u) > uint32(t-1));
 		for (LitVec::size_type i = 0; i < size; ++i, ++first) {
 			score_[first->var()].second += 1 - (first->sign() + first->sign());
 			if (upAct) {
