@@ -434,7 +434,7 @@ bool ParallelSolve::terminate() {
 
 // tries to get new work for the given solver
 bool ParallelSolve::requestWork(Solver& s, PathPtr& out) { 
-	const LitVec* a;
+	const LitVec* a = 0;
 	while (!shared_->terminate()) {
 		// only clear path and stop conflict - we don't propagate() here
 		// because we would then have to handle any eventual conflicts
@@ -442,7 +442,7 @@ bool ParallelSolve::requestWork(Solver& s, PathPtr& out) {
 			// s has a real top-level conflict - problem is unsat
 			terminate(s, true);
 		}	
-		else if (shared_->workQ.try_pop(a)) {
+		else if (a || shared_->workQ.try_pop(a)) {
 			assert(s.decisionLevel() == 0);
 			// got new work from work-queue
 			out = a;
@@ -584,7 +584,7 @@ bool ParallelSolve::backtrackFromModel(Solver& s) {
 			// we have a race condition with solvers that
 			// are currently blocking on the mutex and we could enumerate 
 			// more models than requested by the user
-			terminate(s, r == Enumerator::enumerate_stop_complete);
+			terminate(s, s.decisionLevel() == 0);
 		}
 		if (shared_->enumerator()->enumerated == 1 && !shared_->enumerator()->supportsRestarts()) {
 			// switch to backtracking based splitting algorithm
