@@ -340,7 +340,8 @@ bool Solver::clearAssumptions()  {
 
 void Solver::clearStopConflict() {
 	if (hasConflict() && hasStopConflict()) {
-		rootLevel_ = conflict_[1].asUint();
+		rootLevel_    = conflict_[1].asUint();
+		assign_.front = conflict_[2].asUint();
 		conflict_.clear();
 	}
 }
@@ -354,6 +355,8 @@ void Solver::setStopConflict() {
 		conflict_.push_back(negLit(0));
 		// remember the current root-level
 		conflict_.push_back(Literal::fromRep(rootLevel_));
+		// remember the current propagation queue
+		conflict_.push_back(Literal::fromRep(assign_.front));
 	}
 	// artificially increase root level -
 	// this way, the solver is prevented from resolving the conflict
@@ -776,6 +779,7 @@ uint32 Solver::estimateBCP(const Literal& p, int rd) const {
 
 uint32 Solver::inDegree(WeightLitVec& out) {
 	if (decisionLevel() == 0) { return 1; }
+	assert(!hasConflict());
 	out.reserve((numAssignedVars()-levelStart(1))/10);
 	uint32 maxIn  = 1;
 	uint32 i      = assign_.trail.size(), stop = levelStart(1);
@@ -785,7 +789,6 @@ uint32 Solver::inDegree(WeightLitVec& out) {
 		xAnte         = assign_.reason(x.var());
 		uint32  xIn   = 0;
 		if (!xAnte.isNull() && xAnte.type() != Antecedent::binary_constraint) {
-			conflict_.clear();
 			xAnte.reason(*this, x, conflict_);
 			for (LitVec::const_iterator it = conflict_.begin(); it != conflict_.end(); ++it) {
 				xIn += level(it->var()) != xLev;
@@ -794,8 +797,10 @@ uint32 Solver::inDegree(WeightLitVec& out) {
 				out.push_back(WeightLiteral(x, xIn));
 				maxIn     = std::max(xIn, maxIn);
 			}
+			conflict_.clear();
 		}
 	}
+	assert(!hasConflict());
 	return maxIn;
 }
 /////////////////////////////////////////////////////////////////////////////////////////
