@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2010, Benjamin Kaufmann
+// Copyright (c) 2006-2011, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -76,7 +76,7 @@ bool WeightLits::unique() const {
 // WeightConstraint
 /////////////////////////////////////////////////////////////////////////////////////////
 bool WeightConstraint::newWeightConstraint(SharedContext& ctx, Literal con, WeightLitVec& lits, weight_t bound, Constraint** out) {
-	assert(ctx.master()->decisionLevel() == 0 && ctx.unique());
+	assert(ctx.master()->decisionLevel() == 0 && !ctx.isShared());
 	if (out) *out = 0;
 	weight_t sumW = canonicalize(*ctx.master(), lits, bound);
 	if      (bound <= 0)  { return ctx.addUnary(con); } // trivially SAT
@@ -117,7 +117,7 @@ bool WeightConstraint::newWeightConstraint(SharedContext& ctx, Literal con, Weig
 	uint32 nb   = sizeof(WeightConstraint) + (size+hasW)*sizeof(UndoInfo);
 	void*  m    = 0;
 	WL*    sL   = 0;
-	if (!ctx.shareConstraints()) {
+	if (!ctx.physicalShareProblem()) {
 		nb        = ((nb + 3) / 4)*4;
 		uint32 ls = sizeof(WL) + (size << uint32(hasW))*sizeof(Literal);
 		m         = ::operator new (nb + ls);
@@ -234,7 +234,7 @@ WeightConstraint::WeightConstraint(SharedContext& ctx, Literal W, const WeightLi
 	// init heuristic
 	uint32 off = active_ != NOT_ACTIVE;
 	h[0]       = W;
-	s.strategies().heuristic->newConstraint(s, h+off, lits.size()+(1-off), Constraint_t::static_constraint);
+	s.heuristic()->newConstraint(s, h+off, lits.size()+(1-off), Constraint_t::static_constraint);
 	// init undo stack
 	up_                 = undoStart();     // undo stack is initially empty
 	undo_[0].data       = 0;
@@ -271,7 +271,7 @@ WeightConstraint::WeightConstraint(Solver& s, const WeightConstraint& other) {
 	}
 	// Initialize heuristic with literals (no weights) in constraint.
 	uint32 off = active_ != NOT_ACTIVE;
-	s.strategies().heuristic->newConstraint(s, heu+off, size()-off, Constraint_t::static_constraint);
+	s.heuristic()->newConstraint(s, heu+off, size()-off, Constraint_t::static_constraint);
 	// Init undo stack
 	std::memcpy(undo_, other.undo_, sizeof(UndoInfo)*(size()+isWeight()));
 	up_ = other.up_;
