@@ -17,18 +17,15 @@
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
-
-#include <cppunit/TestFixture.h>
-#include <cppunit/TestAssert.h>
-#include <cppunit/extensions/HelperMacros.h>
-#include <algorithm>
+#include "test.h"
 #include <clasp/weight_constraint.h>
 #include <clasp/solver.h>
+#include <algorithm>
 using namespace std;
 
 namespace Clasp { namespace Test {
-class CardTest : public CppUnit::TestFixture {
-	CPPUNIT_TEST_SUITE(CardTest);
+class WeightConstraintTest : public CppUnit::TestFixture {
+	CPPUNIT_TEST_SUITE(WeightConstraintTest);
 	CPPUNIT_TEST(testAssertTriviallySat);
 	CPPUNIT_TEST(testAssertTriviallyUnSat);
 	CPPUNIT_TEST(testAssertNotSoTriviallySat);
@@ -71,7 +68,7 @@ class CardTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testCloneWeightShared);
 	CPPUNIT_TEST_SUITE_END(); 
 public:
-	CardTest() {
+	WeightConstraintTest() {
 		body  = posLit(ctx.addVar(Var_t::body_var));
 		a     = posLit(ctx.addVar(Var_t::atom_var));
 		b     = posLit(ctx.addVar(Var_t::atom_var));
@@ -656,5 +653,5 @@ private:
 	Literal body;
 	Literal a, b, c, d, e, f;
 };
-CPPUNIT_TEST_SUITE_REGISTRATION(CardTest);
+CPPUNIT_TEST_SUITE_REGISTRATION(WeightConstraintTest);
 } } 
